<?php
/* Copyright (C) 2004-2018  Laurent Destailleur     <eldy@users.sourceforge.net>
 * Copyright (C) 2018-2019  Nicolas ZABOURI         <info@inovea-conseil.com>
 * Copyright (C) 2019-2020  Frédéric France         <frederic.france@netlogic.fr>
 * Copyright (C) 2023 SuperAdmin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * 	\defgroup   clientpart     Module ClientPart
 *  \brief      ClientPart module descriptor.
 *
 *  \file       htdocs/clientpart/core/modules/modClientPart.class.php
 *  \ingroup    clientpart
 *  \brief      Description and activation file for module ClientPart
 */
include_once DOL_DOCUMENT_ROOT.'/core/modules/DolibarrModules.class.php';

/**
 *  Description and activation class for module ClientPart
 */
class modClientPart extends DolibarrModules
{
	/**
	 * Constructor. Define names, constants, directories, boxes, permissions
	 *
	 * @param DoliDB $db Database handler
	 */

	public function __construct($db)
	{
		global $langs, $conf;
		$this->db = $db;

		// Id for module (must be unique).
		// Use here a free id (See in Home -> System information -> Dolibarr for list of used modules id).
		$this->numero = 424050; // TODO Go on page https://wiki.dolibarr.org/index.php/List_of_modules_id to reserve an id number for your module

		// Key text used to identify module (for permissions, menus, etc...)
		$this->rights_class = 'clientpart';

		// Family can be 'base' (core modules),'crm','financial','hr','projects','products','ecm','technic' (transverse modules),'interface' (link with external tools),'other','...'
		// It is used to group modules by family in module setup page
		$this->family = "Pragma-Tech";

		// Module position in the family on 2 digits ('01', '10', '20', ...)
		$this->module_position = '90';

		// Gives the possibility for the module, to provide his own family info and position of this family (Overwrite $this->family and $this->module_position. Avoid this)
		//$this->familyinfo = array('myownfamily' => array('position' => '01', 'label' => $langs->trans("MyOwnFamily")));
		// Module label (no space allowed), used if translation string 'ModuleClientPartName' not found (ClientPart is name of module).
		$this->name = preg_replace('/^mod/i', '', get_class($this));

		// Module description, used if translation string 'ModuleClientPartDesc' not found (ClientPart is name of module).
		$this->description = "ClientPartDescription";
		// Used only if file README.md and README-LL.md not found.
		$this->descriptionlong = "ClientPartDescription";

		// Author
		$this->editor_name = 'Pragma-Tech';
		$this->editor_url = 'https://www.pragma-tech.fr/';

		// Possible values for version are: 'development', 'experimental', 'dolibarr', 'dolibarr_deprecated' or a version string like 'x.y.z'
		$this->version = '1.4.3';
		// Url to the file with your last numberversion of this module
		//$this->url_last_version = 'http://www.example.com/versionmodule.txt';

		// Key used in llx_const table to save module status enabled/disabled (where CLIENTPART is value of property name of module in uppercase)
		$this->const_name = 'MAIN_MODULE_'.strtoupper($this->name);

		// Name of image file used for this module.
		// If file is in theme/yourtheme/img directory under name object_pictovalue.png, use this->picto='pictovalue'
		// If file is in module/img directory under name object_pictovalue.png, use this->picto='pictovalue@module'
		// To use a supported fa-xxx css style of font awesome, use this->picto='xxx'
		$this->picto = 'fa-user';

		// Define some features supported by module (triggers, login, substitutions, menus, css, etc...)
		$this->module_parts = array(
			// Set this to 1 if module has its own trigger directory (core/triggers)
			'triggers' => 1,
			// Set this to 1 if module has its own login method file (core/login)
			'login' => 0,
			// Set this to 1 if module has its own substitution function file (core/substitutions)
			'substitutions' => 0,
			// Set this to 1 if module has its own menus handler directory (core/menus)
			'menus' => 1,
			// Set this to 1 if module overwrite template dir (core/tpl)
			'tpl' => 0,
			// Set this to 1 if module has its own barcode directory (core/modules/barcode)
			'barcode' => 0,
			// Set this to 1 if module has its own models directory (core/modules/xxx)
			'models' => 1,
			// Set this to 1 if module has its own printing directory (core/modules/printing)
			'printing' => 0,
			// Set this to 1 if module has its own theme directory (theme)
			'theme' => 0,
			// Set this to relative path of css file if module has its own css file
			'css' => array(
				//    '/clientpart/css/clientpart.css.php',
			),
			// Set this to relative path of js file if module must load a js on all pages
			'js' => array(
				//   '/clientpart/js/clientpart.js.php',
			),
			// Set here all hooks context managed by module. To find available hook context, make a "grep -r '>initHooks(' *" on source code. You can also set hook context to 'all'
			'hooks' => array(
				//   'data' => array(
				//       'hookcontext1',
				//       'hookcontext2',
				//   ),
				//   'entity' => '0',
			),
			// Set this to 1 if features of module are opened to external users
			'moduleforexternal' => 0,
		);

		// Data directories to create when module is enabled.
		// Example: this->dirs = array("/clientpart/temp","/clientpart/subdir");
		$this->dirs = array("/clientpart/temp");

		// Config pages. Put here list of php page, stored into clientpart/admin directory, to use to setup module.
		$this->config_page_url = array("setup.php@clientpart");

		// Dependencies
		// A condition to hide module
		$this->hidden = false;
		// List of module class names as string that must be enabled if this module is enabled. Example: array('always1'=>'modModuleToEnable1','always2'=>'modModuleToEnable2', 'FR1'=>'modModuleToEnableFR'...)
		$this->depends = array();
		$this->requiredby = array(); // List of module class names as string to disable if this one is disabled. Example: array('modModuleToDisable1', ...)
		$this->conflictwith = array(); // List of module class names as string this module is in conflict with. Example: array('modModuleToDisable1', ...)

		// The language file dedicated to your module
		$this->langfiles = array("clientpart@clientpart");

		// Prerequisites
		$this->phpmin = array(5, 6); // Minimum version of PHP required by module
		$this->need_dolibarr_version = array(11, -3); // Minimum version of Dolibarr required by module

		// Messages at activation
		$this->warnings_activation = array(); // Warning to show when we activate module. array('always'='text') or array('FR'='textfr','MX'='textmx'...)
		$this->warnings_activation_ext = array(); // Warning to show when we activate an external module. array('always'='text') or array('FR'='textfr','MX'='textmx'...)
		//$this->automatic_activation = array('FR'=>'ClientPartWasAutomaticallyActivatedBecauseOfYourCountryChoice');
		//$this->always_enabled = true;								// If true, can't be disabled

		// Constants
		// List of particular constants to add when module is enabled (key, 'chaine', value, desc, visible, 'current' or 'allentities', deleteonunactive)
		// Example: $this->const=array(1 => array('CLIENTPART_MYNEWCONST1', 'chaine', 'myvalue', 'This is a constant to add', 1),
		//                             2 => array('CLIENTPART_MYNEWCONST2', 'chaine', 'myvalue', 'This is another constant to add', 0, 'current', 1)
		// );
		$this->const = array();

		// Some keys to add into the overwriting translation tables
		/*$this->overwrite_translation = array(
			'en_US:ParentCompany'=>'Parent company or reseller',
			'fr_FR:ParentCompany'=>'Maison mère ou revendeur'
		)*/

		if (!isset($conf->clientpart) || !isset($conf->clientpart->enabled)) {
			$conf->clientpart = new stdClass();
			$conf->clientpart->enabled = 0;
		}

		// Array to add new pages in new tabs
		$this->tabs = array();
		// Example:
		// $this->tabs[] = array('data'=>'objecttype:+tabname1:Title1:mylangfile@clientpart:$user->rights->clientpart->read:/clientpart/mynewtab1.php?id=__ID__');  					// To add a new tab identified by code tabname1
		// $this->tabs[] = array('data'=>'objecttype:+tabname2:SUBSTITUTION_Title2:mylangfile@clientpart:$user->rights->othermodule->read:/clientpart/mynewtab2.php?id=__ID__',  	// To add another new tab identified by code tabname2. Label will be result of calling all substitution functions on 'Title2' key.
		// $this->tabs[] = array('data'=>'objecttype:-tabname:NU:conditiontoremove');                                                     										// To remove an existing tab identified by code tabname
		//
		// Where objecttype can be
		// 'categories_x'	  to add a tab in category view (replace 'x' by type of category (0=product, 1=supplier, 2=customer, 3=member)
		// 'contact'          to add a tab in contact view
		// 'contract'         to add a tab in contract view
		// 'group'            to add a tab in group view
		// 'intervention'     to add a tab in intervention view
		// 'invoice'          to add a tab in customer invoice view
		// 'invoice_supplier' to add a tab in supplier invoice view
		// 'member'           to add a tab in fundation member view
		// 'opensurveypoll'	  to add a tab in opensurvey poll view
		// 'order'            to add a tab in customer order view
		// 'order_supplier'   to add a tab in supplier order view
		// 'payment'		  to add a tab in payment view
		// 'payment_supplier' to add a tab in supplier payment view
		// 'product'          to add a tab in product view
		// 'propal'           to add a tab in propal view
		// 'project'          to add a tab in project view
		// 'stock'            to add a tab in stock view
		// 'thirdparty'       to add a tab in third party view
		// 'user'             to add a tab in user view

		// Dictionaries
		$this->dictionaries = array();
		/* Example:
		$this->dictionaries=array(
			'langs'=>'clientpart@clientpart',
			// List of tables we want to see into dictonnary editor
			'tabname'=>array("table1", "table2", "table3"),
			// Label of tables
			'tablib'=>array("Table1", "Table2", "Table3"),
			// Request to select fields
			'tabsql'=>array('SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table1 as f', 'SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table2 as f', 'SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table3 as f'),
			// Sort order
			'tabsqlsort'=>array("label ASC", "label ASC", "label ASC"),
			// List of fields (result of select to show dictionary)
			'tabfield'=>array("code,label", "code,label", "code,label"),
			// List of fields (list of fields to edit a record)
			'tabfieldvalue'=>array("code,label", "code,label", "code,label"),
			// List of fields (list of fields for insert)
			'tabfieldinsert'=>array("code,label", "code,label", "code,label"),
			// Name of columns with primary key (try to always name it 'rowid')
			'tabrowid'=>array("rowid", "rowid", "rowid"),
			// Condition to show each dictionary
			'tabcond'=>array($conf->clientpart->enabled, $conf->clientpart->enabled, $conf->clientpart->enabled),
			// Tooltip for every fields of dictionaries: DO NOT PUT AN EMPTY ARRAY
			'tabhelp'=>array(array('code'=>$langs->trans('CodeTooltipHelp'), 'field2' => 'field2tooltip'), array('code'=>$langs->trans('CodeTooltipHelp'), 'field2' => 'field2tooltip'), ...),
		);
		*/

		// Boxes/Widgets
		// Add here list of php file(s) stored in clientpart/core/boxes that contains a class to show a widget.
		$this->boxes = array(
			//  0 => array(
			//      'file' => 'clientpartwidget1.php@clientpart',
			//      'note' => 'Widget provided by ClientPart',
			//      'enabledbydefaulton' => 'Home',
			//  ),
			//  ...
		);

		// Cronjobs (List of cron jobs entries to add when module is enabled)
		// unit_frequency must be 60 for minute, 3600 for hour, 86400 for day, 604800 for week
		$this->cronjobs = array(
			//  0 => array(
			//      'label' => 'MyJob label',
			//      'jobtype' => 'method',
			//      'class' => '/clientpart/class/myobject.class.php',
			//      'objectname' => 'MyObject',
			//      'method' => 'doScheduledJob',
			//      'parameters' => '',
			//      'comment' => 'Comment',
			//      'frequency' => 2,
			//      'unitfrequency' => 3600,
			//      'status' => 0,
			//      'test' => '$conf->clientpart->enabled',
			//      'priority' => 50,
			//  ),
		);
		// Example: $this->cronjobs=array(
		//    0=>array('label'=>'My label', 'jobtype'=>'method', 'class'=>'/dir/class/file.class.php', 'objectname'=>'MyClass', 'method'=>'myMethod', 'parameters'=>'param1, param2', 'comment'=>'Comment', 'frequency'=>2, 'unitfrequency'=>3600, 'status'=>0, 'test'=>'$conf->clientpart->enabled', 'priority'=>50),
		//    1=>array('label'=>'My label', 'jobtype'=>'command', 'command'=>'', 'parameters'=>'param1, param2', 'comment'=>'Comment', 'frequency'=>1, 'unitfrequency'=>3600*24, 'status'=>0, 'test'=>'$conf->clientpart->enabled', 'priority'=>50)
		// );

		// Permissions provided by this module
		$this->rights = array();
		$r = 0;
		// Add here entries to declare new permissions
		/* BEGIN MODULEBUILDER PERMISSIONS */
		$this->rights[$r][0] = $this->numero . sprintf("%02d", $r + 1); // Permission id (must not be already used)
		$this->rights[$r][1] = $langs->trans('CLIENTPART_PERM_READ'); // Permission label
		$this->rights[$r][4] = 'particulier';
		$this->rights[$r][5] = 'read'; // In php code, permission will be checked by test if ($user->rights->clientpart->myobject->read)
		$r++;
		$this->rights[$r][0] = $this->numero . sprintf("%02d", $r + 1); // Permission id (must not be already used)
		$this->rights[$r][1] = $langs->trans('CLIENTPART_PERM_WRITE'); // Permission label
		$this->rights[$r][4] = 'particulier';
		$this->rights[$r][5] = 'write'; // In php code, permission will be checked by test if ($user->rights->clientpart->myobject->write)
		$r++;
		$this->rights[$r][0] = $this->numero . sprintf("%02d", $r + 1); // Permission id (must not be already used)
		$this->rights[$r][1] = $langs->trans('CLIENTPART_PERM_DELETE'); // Permission label
		$this->rights[$r][4] = 'particulier';
		$this->rights[$r][5] = 'delete'; // In php code, permission will be checked by test if ($user->rights->clientpart->myobject->delete)
		$r++;
		/* END MODULEBUILDER PERMISSIONS */

		// Main menu entries to add
		$this->menu = array();
		$r = 0;
		// Add here entries to declare new menus
		/* BEGIN MODULEBUILDER TOPMENU */
		$this->menu[$r++] = array(
			'fk_menu'=>'fk_mainmenu=companies,fk_leftmenu=companies_particulier_list', // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
			'type'=>'left', // This is a Top menu entry
			'titre'=>$langs->trans('LeftMenuPrivateCustomersForm'),
			'prefix' => img_picto('', $this->picto, 'class="paddingright pictofixedwidth valignmiddle"'),
			'mainmenu'=>'companies',
			'leftmenu'=>'',
			'url'=>'/clientpart/clientpartindex.php',
			'langs'=>'clientpart@clientpart', // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
			'position'=>2,
			'enabled'=>'$conf->clientpart->enabled', // Define condition to show or hide menu entry. Use '$conf->testmenuhook->enabled' if entry must be visible if module is enabled.
			'perms'=>'1', // Use 'perms'=>'$user->rights->testmenuhook->myobject->read' if you want your menu with a permission rules
			'target'=>'',
			'user'=>2, // 0=Menu for internal users, 1=external users, 2=both
		);

		$this->menu[$r++]=array(
			'fk_menu'=>'fk_mainmenu=companies,fk_leftmenu=thirdparties',	    // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
			'type'=>'left',			                // This is a Left menu entry
			'titre'=>$langs->trans('LeftMenuPrivateCustomersList'),
			'mainmenu'=>'companies',
			'leftmenu'=>'companies_particulier_list',
			'url'=>'/societe/list.php?sortfield=s.nom&sortorder=ASC&contextpage=thirdpartylist&limit=20&search_categ_cus=-1&search_categ_sup=-1&search_sale=-1&search_type_thirdparty=8&search_type=-1&search_options_clientpart_civilite=0&search_status=1',
			'langs'=>'clientpart@clientpart',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
			'position'=>1,
			'enabled'=>'$conf->clientpart->enabled',  // Define condition to show or hide menu entry. Use '$conf->clientpart->enabled' if entry must be visible if module is enabled. Use '$leftmenu==\'system\'' to show if leftmenu system is selected.
			'perms'=>'1',			                // Use 'perms'=>'$user->rights->clientpart->level1->level2' if you want your menu with a permission rules
			'target'=>'',
			'user'=>2,				                // 0=Menu for internal users, 1=external users, 2=both
		);


		/* END MODULEBUILDER TOPMENU */
		/* BEGIN MODULEBUILDER LEFTMENU MYOBJECT*/
//		$this->menu[$r++]=array(
//			'fk_menu'=>'fk_mainmenu=clientpart',      // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
//			'type'=>'left',                          // This is a Left menu entry
//			'titre'=>'MyObject',
//			'prefix' => img_picto('', $this->picto, 'class="paddingright pictofixedwidth valignmiddle"'),
//			'mainmenu'=>'clientpart',
//			'leftmenu'=>'myobject',
//			'url'=>'/clientpart/clientpartindex.php',
//			'langs'=>'clientpart@clientpart',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
//			'position'=>1000+$r,
//			'enabled'=>'$conf->clientpart->enabled',  // Define condition to show or hide menu entry. Use '$conf->clientpart->enabled' if entry must be visible if module is enabled.
//			'perms'=>'$user->rights->clientpart->myobject->read',			                // Use 'perms'=>'$user->rights->clientpart->level1->level2' if you want your menu with a permission rules
//			'target'=>'',
//			'user'=>2,				                // 0=Menu for internal users, 1=external users, 2=both
//		);

//		$this->menu[$r++]=array(
//			'fk_menu'=>'fk_mainmenu=clientpart,fk_leftmenu=myobject',	    // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
//			'type'=>'left',			                // This is a Left menu entry
//			'titre'=>'New_MyObject',
//			'mainmenu'=>'clientpart',
//			'leftmenu'=>'clientpart_myobject_new',
//			'url'=>'/clientpart/myobject_card.php?action=create',
//			'langs'=>'clientpart@clientpart',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
//			'position'=>1000+$r,
//			'enabled'=>'$conf->clientpart->enabled',  // Define condition to show or hide menu entry. Use '$conf->clientpart->enabled' if entry must be visible if module is enabled. Use '$leftmenu==\'system\'' to show if leftmenu system is selected.
//			'perms'=>'$user->rights->clientpart->myobject->write',			                // Use 'perms'=>'$user->rights->clientpart->level1->level2' if you want your menu with a permission rules
//			'target'=>'',
//			'user'=>2,				                // 0=Menu for internal users, 1=external users, 2=both
//		);
		/*END MODULEBUILDER LEFTMENU MYOBJECT */
		// Exports profiles provided by this module
		$r = 1;
		/* BEGIN MODULEBUILDER EXPORT MYOBJECT */
		/*
		$langs->load("clientpart@clientpart");
		$this->export_code[$r]=$this->rights_class.'_'.$r;
		$this->export_label[$r]='MyObjectLines';	// Translation key (used only if key ExportDataset_xxx_z not found)
		$this->export_icon[$r]='myobject@clientpart';
		// Define $this->export_fields_array, $this->export_TypeFields_array and $this->export_entities_array
		$keyforclass = 'MyObject'; $keyforclassfile='/clientpart/class/myobject.class.php'; $keyforelement='myobject@clientpart';
		include DOL_DOCUMENT_ROOT.'/core/commonfieldsinexport.inc.php';
		//$this->export_fields_array[$r]['t.fieldtoadd']='FieldToAdd'; $this->export_TypeFields_array[$r]['t.fieldtoadd']='Text';
		//unset($this->export_fields_array[$r]['t.fieldtoremove']);
		//$keyforclass = 'MyObjectLine'; $keyforclassfile='/clientpart/class/myobject.class.php'; $keyforelement='myobjectline@clientpart'; $keyforalias='tl';
		//include DOL_DOCUMENT_ROOT.'/core/commonfieldsinexport.inc.php';
		$keyforselect='myobject'; $keyforaliasextra='extra'; $keyforelement='myobject@clientpart';
		include DOL_DOCUMENT_ROOT.'/core/extrafieldsinexport.inc.php';
		//$keyforselect='myobjectline'; $keyforaliasextra='extraline'; $keyforelement='myobjectline@clientpart';
		//include DOL_DOCUMENT_ROOT.'/core/extrafieldsinexport.inc.php';
		//$this->export_dependencies_array[$r] = array('myobjectline'=>array('tl.rowid','tl.ref')); // To force to activate one or several fields if we select some fields that need same (like to select a unique key if we ask a field of a child to avoid the DISTINCT to discard them, or for computed field than need several other fields)
		//$this->export_special_array[$r] = array('t.field'=>'...');
		//$this->export_examplevalues_array[$r] = array('t.field'=>'Example');
		//$this->export_help_array[$r] = array('t.field'=>'FieldDescHelp');
		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'myobject as t';
		//$this->export_sql_end[$r]  =' LEFT JOIN '.MAIN_DB_PREFIX.'myobject_line as tl ON tl.fk_myobject = t.rowid';
		$this->export_sql_end[$r] .=' WHERE 1 = 1';
		$this->export_sql_end[$r] .=' AND t.entity IN ('.getEntity('myobject').')';
		$r++; */
		/* END MODULEBUILDER EXPORT MYOBJECT */

		// Imports profiles provided by this module
		$r = 1;
		/* BEGIN MODULEBUILDER IMPORT MYOBJECT */
		/*
		$langs->load("clientpart@clientpart");
		$this->import_code[$r]=$this->rights_class.'_'.$r;
		$this->import_label[$r]='MyObjectLines';	// Translation key (used only if key ExportDataset_xxx_z not found)
		$this->import_icon[$r]='myobject@clientpart';
		$this->import_tables_array[$r] = array('t' => MAIN_DB_PREFIX.'clientpart_myobject', 'extra' => MAIN_DB_PREFIX.'clientpart_myobject_extrafields');
		$this->import_tables_creator_array[$r] = array('t' => 'fk_user_author'); // Fields to store import user id
		$import_sample = array();
		$keyforclass = 'MyObject'; $keyforclassfile='/clientpart/class/myobject.class.php'; $keyforelement='myobject@clientpart';
		include DOL_DOCUMENT_ROOT.'/core/commonfieldsinimport.inc.php';
		$import_extrafield_sample = array();
		$keyforselect='myobject'; $keyforaliasextra='extra'; $keyforelement='myobject@clientpart';
		include DOL_DOCUMENT_ROOT.'/core/extrafieldsinimport.inc.php';
		$this->import_fieldshidden_array[$r] = array('extra.fk_object' => 'lastrowid-'.MAIN_DB_PREFIX.'clientpart_myobject');
		$this->import_regex_array[$r] = array();
		$this->import_examplevalues_array[$r] = array_merge($import_sample, $import_extrafield_sample);
		$this->import_updatekeys_array[$r] = array('t.ref' => 'Ref');
		$this->import_convertvalue_array[$r] = array(
			't.ref' => array(
				'rule'=>'getrefifauto',
				'class'=>(empty($conf->global->CLIENTPART_MYOBJECT_ADDON) ? 'mod_myobject_standard' : $conf->global->CLIENTPART_MYOBJECT_ADDON),
				'path'=>"/core/modules/commande/".(empty($conf->global->CLIENTPART_MYOBJECT_ADDON) ? 'mod_myobject_standard' : $conf->global->CLIENTPART_MYOBJECT_ADDON).'.php'
				'classobject'=>'MyObject',
				'pathobject'=>'/clientpart/class/myobject.class.php',
			),
			't.fk_soc' => array('rule' => 'fetchidfromref', 'file' => '/societe/class/societe.class.php', 'class' => 'Societe', 'method' => 'fetch', 'element' => 'ThirdParty'),
			't.fk_user_valid' => array('rule' => 'fetchidfromref', 'file' => '/user/class/user.class.php', 'class' => 'User', 'method' => 'fetch', 'element' => 'user'),
			't.fk_mode_reglement' => array('rule' => 'fetchidfromcodeorlabel', 'file' => '/compta/paiement/class/cpaiement.class.php', 'class' => 'Cpaiement', 'method' => 'fetch', 'element' => 'cpayment'),
		);
		$r++; */
		/* END MODULEBUILDER IMPORT MYOBJECT */
	}

	/**
	 *  Function called when module is enabled.
	 *  The init function add constants, boxes, permissions and menus (defined in constructor) into Dolibarr database.
	 *  It also creates data directories
	 *
	 *  @param      string  $options    Options when enabling module ('', 'noboxes')
	 *  @return     int             	1 if OK, 0 if KO
	 */
	public function init($options = '')
	{
		global $conf, $langs;

		//$result = $this->_load_tables('/install/mysql/', 'clientpart');
		$result = $this->_load_tables('/clientpart/sql/');
		if ($result < 0) {
			return -1; // Do not activate module if error 'not allowed' returned when loading module SQL queries (the _load_table run sql with run_sql with the error allowed parameter set to 'default')
		}

		// Create extrafields during init
		include_once DOL_DOCUMENT_ROOT.'/core/class/extrafields.class.php';

		$extrafields = new ExtraFields($this->db);

		// <Pragma>
		$paramPartner = 'a:1:{s:7:"options";a:1:{s:140:"socpeople LEFT JOIN '.MAIN_DB_PREFIX.'c_civility ON '.MAIN_DB_PREFIX.'c_civility.code = '.MAIN_DB_PREFIX.'socpeople.civility:label|firstname|lastname:'.MAIN_DB_PREFIX.'socpeople.rowid::fk_soc=$ID$";N;}}';

		$result1=$extrafields->addExtraField('clientpart_firstname', "LABEL_LASTNAME", 'varchar', 1,  20, 'thirdparty',   0, 0, '', '', 1, '', 3, 0, '', '', 'clientpart@clientpart', '$conf->clientpart->enabled');
		$result4=$extrafields->addExtraField('clientpart_civilite', "LABEL_CIVILITY", 'sellist',  1,  '', 'thirdparty',   0, 0, '', array('options'=>array('c_civility:label:rowid'=>'c_civility:label:rowid')), 1,'', 3, 0, '', '', 'clientpart@clientpart', '$conf->clientpart->enabled');
		$result4=$extrafields->addExtraField('clientpart_partner', "LABEL_BETROTHED", 'sellist',  1,  '', 'thirdparty',   0, 0, '', $paramPartner, 1,'', 3, "HelpFirstName", '', '', 'clientpart@clientpart', '$conf->clientpart->enabled');
		//$result4=$extrafields->update('clientpart_partner', "LABEL_BETROTHED", 'sellist',  1,  '', 'thirdparty',   0, 0, '', $paramPartner, 1,'', 3, "HelpFirstName", '', '', 'clientpart@clientpart', '$conf->clientpart->enabled');

		$sql = "SELECT * FROM ".MAIN_DB_PREFIX."extrafields as ex WHERE ex.name='clientpart_partner'";
		$result=$this->db->query($sql);

		if (!empty($result->num_rows)) {

			foreach($result as $row) {

				if ($row['param']<>$paramPartner) {

					$sql="Update ".MAIN_DB_PREFIX."extrafields SET param='".$paramPartner."' WHERE name='clientpart_partner'";
					$this->db->query($sql);

				}

			}

		}

		// </Pragma>

		// Permissions
		$this->remove($options);

		// <Pragma>

		// PDF Model Management
		$myDocsActiveVerifications = array (
			// Propal
			array(
				// Type of PDF Model
				'type' => 'propal',
				// Name of PDF Model
				'name' => 'cyan_clientpart',
				// Default PDF Const name
				'const' => 'PROPALE_ADDON_PDF'
			),
			// Order
			array(
				// Type of PDF Model
				'type' => 'order',
				// Name of PDF Model
				'name' => 'eratosthene_clientpart',
				// Default PDF Const name
				'const' => 'COMMANDE_ADDON_PDF'
			),
			// Expedition
			array(
				// Type of PDF Model
				'type' => 'shipping',
				// Name of PDF Model
				'name' => 'espadon_clientpart',
				// Default PDF Const name
				'const' => 'EXPEDITION_ADDON_PDF'
			),
			// Delivery
			array(
				// Type of PDF Model
				'type' => 'delivery',
				// Name of PDF Model
				'name' => 'typhon_clientpart',
				// Default PDF Const name
				'const' => 'DELIVERY_ADDON_PDF'
			),
			// Invoice
			array(
				// Type of PDF Model
				'type' => 'invoice',
				// Name of PDF Model
				'name' => 'sponge_clientpart',
				// Default PDF Const name
				'const' => 'FACTURE_ADDON_PDF'
			),
			// CONTRACT
			array(
				// Type of PDF Model
				'type' => 'contract',
				// Name of PDF Model
				'name' => 'strato_clientpart',
				// Default PDF Const name
				'const' => 'CONTRACT_ADDON_PDF'
			),
			// INTERVENTION
			array(
				// Type of PDF Model
				'type' => 'ficheinter',
				// Name of PDF Model
				'name' => 'soleil_clientpart',
				// Default PDF Const name
				'const' => 'FICHEINTER_ADDON_PDF'
			),
			// EXPENSEREPORT
			array(
				// Type of PDF Model
				'type' => 'expensereport',
				// Name of PDF Model
				'name' => 'standard_clientpart',
				// Default PDF Const name
				'const' => 'EXPENSEREPORT_ADDON_PDF'
			)
		);

		foreach ($myDocsActiveVerifications as $currentDocActiveVerification) {

			$sqlDocVerification="SELECT * FROM ".MAIN_DB_PREFIX."document_model WHERE nom = '".$currentDocActiveVerification['name']."' AND type = '".$currentDocActiveVerification['type']."' ";

			$resultDocVerification = $this->db->query($sqlDocVerification);

				if ( $resultDocVerification->num_rows >0) {
					$sqlDocDelete="DELETE FROM ".MAIN_DB_PREFIX."document_model WHERE nom = '".$currentDocActiveVerification['name']."' AND type = '".$currentDocActiveVerification['type']."' ";
					$resultDelete=$this->db->query($sqlDocDelete);

					$sqlDocInsert="INSERT INTO ".MAIN_DB_PREFIX."document_model (nom, type, entity, libelle, description) VALUES ('".$currentDocActiveVerification['name']."','".$currentDocActiveVerification['type']."',1, '".$currentDocActiveVerification['name']."', null)";
					$resultDocInsert=$this->db->query($sqlDocInsert);
					if (!empty($conf->global->CLIENTPART_PDF_DEFAULT_ALL)) {

						$sqlDocDefaultInsert="UPDATE ".MAIN_DB_PREFIX."const SET value = '".$currentDocActiveVerification['name']."' WHERE name='".$currentDocActiveVerification['const']."'";
						$resultDocInsert=$this->db->query($sqlDocDefaultInsert);

					}

				} else {
					$sqlDocInsert="INSERT INTO ".MAIN_DB_PREFIX."document_model (nom, type, entity, libelle, description) VALUES ('".$currentDocActiveVerification['name']."','".$currentDocActiveVerification['type']."',1, '".$currentDocActiveVerification['name']."', null)";
					$resultDocInsert=$this->db->query($sqlDocInsert);

					if (!empty($conf->global->CLIENTPART_PDF_DEFAULT_ALL)) {

						$sqlDocDefaultInsert="UPDATE ".MAIN_DB_PREFIX."const SET value = '".$currentDocActiveVerification['name']."' WHERE name='".$currentDocActiveVerification['const']."'";
						$resultDocInsert=$this->db->query($sqlDocDefaultInsert);

					}

				}

		}

		$myCivility = array (
			array(
				"code" => "MME2",
				"label" => "Mme",
				"active" => 1,
			),
			array(
				"code" => "MR2",
				"label" => "M.",
				"active" => 1,
			),
			array(
				"code" => "MTRE2",
				"label" => "Me",
				"active" => 1,
			)
		);

		foreach ($myCivility as $currentCivility) {

			$sqlCivilityVerification = "SELECT * FROM `llx_c_civility` as civ WHERE civ.code='".$currentCivility['code']."'";
			$resultCivilityVerification=$this->db->query($sqlCivilityVerification);

			if (empty($resultCivilityVerification->num_rows)) {
				$sqlCivilityInsert = "INSERT INTO `llx_c_civility`(`code`, `label`, `active`,`module`) VALUES ('".$currentCivility['code']."','".$currentCivility['label']."',".$currentCivility['active'].",'clientpart')";
				$resultCivilityInsert=$this->db->query($sqlCivilityInsert);
			}

		}

		/*$myMustColumn = array (
			array(
				'URL' => 'contact/card.php?action=create',
				'HTMLName' => 'firstname',
				'type' => 'mandatory'
			),
			array(
				'URL' => 'contact/card.php?action=edit',
				'HTMLName' => 'firstname',
				'type' => 'mandatory'
			)
		);

		// CLIENTPART_MUST_FIRSTNAME
		if (!empty($conf->global->CLIENTPART_MUST_FIRSTNAME)) {


			foreach ($myMustColumn as $currentMustColumn) {
				// INSERT INTO llx_default_values (type, user_id, page, param, value) VALUES ('mandatory', 0, 'contact/card.php?action=edit', 'firstname', '')
				// "INSERT INTO `llx_default_values`(`type`, `user_id`, `page`, `param`) VALUES ('mandatory',0,'".$currentMustColumn['URL']."','".$currentMustColumn['HTMLName']."')";

				$sqlMustColumnVerification = "SELECT * FROM `llx_default_values` as defaultV";
				$sqlMustColumnVerification .= " WHERE defaultV.type='".$currentMustColumn['type']."' AND defaultV.param='".$currentMustColumn['HTMLName']."' AND defaultV.page='".$currentMustColumn['URL']."'";

				$resultMustColumnVerification=$this->db->query($sqlMustColumnVerification);

				if (empty($resultMustColumnVerification->num_rows)) {
					$sqlMustColumnInsert = "INSERT INTO `llx_default_values`(`type`, `user_id`, `page`, `param`) VALUES ('".$currentMustColumn['type']."',0,'".$currentMustColumn['URL']."','".$currentMustColumn['HTMLName']."')";
					$resultMustColumnInsert=$this->db->query($sqlMustColumnInsert);
				}

			}
		} else {

			foreach ($myMustColumn as $currentMustColumn) {
				// INSERT INTO llx_default_values (type, user_id, page, param, value) VALUES ('mandatory', 0, 'contact/card.php?action=edit', 'firstname', '')
				// "INSERT INTO `llx_default_values`(`type`, `user_id`, `page`, `param`) VALUES ('mandatory',0,'".$currentMustColumn['URL']."','".$currentMustColumn['HTMLName']."')";

				$sqlMustColumnVerification = "SELECT * FROM `".MAIN_DB_PREFIX."default_values` as defaultV";
				$sqlMustColumnVerification .= " WHERE defaultV.type='".$currentMustColumn['type']."' AND defaultV.param='".$currentMustColumn['HTMLName']."' AND defaultV.page='".$currentMustColumn['URL']."'";

				$resultMustColumnVerification=$this->db->query($sqlMustColumnVerification);

				if (!empty($resultMustColumnVerification->num_rows)) {
					$sqlMustColumnDelete = "DELETE FROM `".MAIN_DB_PREFIX."default_values` WHERE defaultV.type='".$currentMustColumn['type']."' AND defaultV.param='".$currentMustColumn['HTMLName']."' AND defaultV.page='".$currentMustColumn['URL']."'";
					$resultMustColumnDelete=$this->db->query($sqlMustColumnDelete);
				}

			}

		}*/


		// </Pragma>
		$sql = array();

		// Document templates
		$moduledir = dol_sanitizeFileName('clientpart');
		$myTmpObjects = array();

		$myTmpObjects['MyObject'] = array('includerefgeneration'=>0, 'includedocgeneration'=>0);


		foreach ($myTmpObjects as $myTmpObjectKey => $myTmpObjectArray) {
			if ($myTmpObjectKey == 'MyObject') {
				continue;
			}
			if ($myTmpObjectArray['includerefgeneration']) {
				$src = DOL_DOCUMENT_ROOT.'/install/doctemplates/'.$moduledir.'/template_myobjects.odt';
//				$src = DOL_DOCUMENT_ROOT.'/install/doctemplates/'.'clientpart'.'/template_myobjects.odt';
				$dirodt = DOL_DATA_ROOT.'/doctemplates/'.$moduledir;
//				$dirodt = DOL_DATA_ROOT.'/doctemplates/'.'clientpart';
				$dest = $dirodt.'/template_myobjects.odt';

				if (file_exists($src) && !file_exists($dest)) {
					print "test<br>";
					require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
					dol_mkdir($dirodt);
					$result = dol_copy($src, $dest, 0, 0);
					if ($result < 0) {
						$langs->load("errors");
						$this->error = $langs->trans('ErrorFailToCopyFile', $src, $dest);
						return 0;
					}
				}


				$sql = array_merge($sql, array(
					"DELETE FROM ".MAIN_DB_PREFIX."document_model WHERE nom = 'standard_".strtolower($myTmpObjectKey)."' AND type = '".$this->db->escape(strtolower($myTmpObjectKey))."' AND entity = ".((int) $conf->entity),
					"INSERT INTO ".MAIN_DB_PREFIX."document_model (nom, type, entity) VALUES('standard_".strtolower($myTmpObjectKey)."', '".$this->db->escape(strtolower($myTmpObjectKey))."', ".((int) $conf->entity).")",
					"DELETE FROM ".MAIN_DB_PREFIX."document_model WHERE nom = 'generic_".strtolower($myTmpObjectKey)."_odt' AND type = '".$this->db->escape(strtolower($myTmpObjectKey))."' AND entity = ".((int) $conf->entity),
					"INSERT INTO ".MAIN_DB_PREFIX."document_model (nom, type, entity) VALUES('generic_".strtolower($myTmpObjectKey)."_odt', '".$this->db->escape(strtolower($myTmpObjectKey))."', ".((int) $conf->entity).")"

				));

			}
		}

		return $this->_init($sql, $options);
	}

	/**
	 *  Function called when module is disabled.
	 *  Remove from database constants, boxes and permissions from Dolibarr database.
	 *  Data directories are not deleted
	 *
	 *  @param      string	$options    Options when enabling module ('', 'noboxes')
	 *  @return     int                 1 if OK, 0 if KO
	 */
	public function remove($options = '')
	{
		/*$myMustColumn = array (
			array(
				'URL' => 'contact/card.php?action=create',
				'HTMLName' => 'firstname',
				'type' => 'mandatory'
			),
			array(
				'URL' => 'contact/card.php?action=edit',
				'HTMLName' => 'firstname',
				'type' => 'mandatory'
			)
		);

		foreach ($myMustColumn as $currentMustColumn) {
			// INSERT INTO llx_default_values (type, user_id, page, param, value) VALUES ('mandatory', 0, 'contact/card.php?action=edit', 'firstname', '')
			// "INSERT INTO `llx_default_values`(`type`, `user_id`, `page`, `param`) VALUES ('mandatory',0,'".$currentMustColumn['URL']."','".$currentMustColumn['HTMLName']."')";

			$sqlMustColumnVerification = "SELECT * FROM `".MAIN_DB_PREFIX."default_values` as defaultV";
			$sqlMustColumnVerification .= " WHERE defaultV.type='".$currentMustColumn['type']."' AND defaultV.param='".$currentMustColumn['HTMLName']."' AND defaultV.page='".$currentMustColumn['URL']."'";

			$resultMustColumnVerification=$this->db->query($sqlMustColumnVerification);

			if (!empty($resultMustColumnVerification->num_rows)) {
				$sqlMustColumnDelete = "DELETE FROM `".MAIN_DB_PREFIX."default_values` as defaultV WHERE defaultV.type='".$currentMustColumn['type']."' AND defaultV.param='".$currentMustColumn['HTMLName']."' AND defaultV.page='".$currentMustColumn['URL']."'";
				echo $sqlMustColumnDelete;
				exit;
				$resultMustColumnDelete=$this->db->query($sqlMustColumnDelete);
			}

		}*/

		$sql = array();
		return $this->_remove($sql, $options);
	}
}
